<schema xmlns="http://purl.oclc.org/dsdl/schematron" queryBinding="xslt2"
        xmlns:sqf="http://www.schematron-quickfix.com/validator/process"
        xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
  <title>Schematron QuickFix Example: Change attribute value with custom XSLT function</title>
  <ns prefix="tei" uri="http://www.tei-c.org/ns/1.0"/>
  <ns prefix="xs" uri="http://www.w3.org/2001/XMLSchema"/>
  <ns prefix="fn" uri="tag:dmaus@dmaus.name,2019:Schematron:Workshop"/>
  <xsl:function name="fn:add-hashbang" as="xs:string*">
    <xsl:param name="tokens" as="xs:string*"/>
    <xsl:for-each select="$tokens">
      <xsl:value-of select="if (starts-with(., '#')) then . else concat('#', .)"/>
    </xsl:for-each>
  </xsl:function>
  <pattern>
    <rule context="tei:rs/@ref">
      <let name="tokens" value="tokenize(., '\s+')"/>
      <assert test="exists($tokens)" sqf:fix="delete-attribute edit-attribute">
        The @ref attribute of a referring string must not be empty
      </assert>
      <assert test="every $token in $tokens satisfies starts-with($token, '#')" sqf:fix="remove-nohash add-hashbang edit-attribute">
        Every value in a @ref attribute of a referring string must start with a hashbang
      </assert>
      <sqf:fix id="remove-nohash">
        <sqf:description>
          <sqf:title>Remove values without a hashbang</sqf:title>
        </sqf:description>
        <sqf:replace node-type="attribute" target="ref" select="$tokens[starts-with(., '#')]"/>
      </sqf:fix>
      <sqf:fix id="add-hashbang">
        <sqf:description>
          <sqf:title>Add hashbang to values without one</sqf:title>
        </sqf:description>
        <sqf:replace node-type="attribute" target="ref" select="fn:add-hashbang($tokens)"/>
      </sqf:fix>
      <sqf:fix id="delete-attribute">
        <sqf:description>
          <sqf:title>Delete the @<name/> attribute</sqf:title>
        </sqf:description>
        <sqf:delete/>
      </sqf:fix>
      <sqf:fix id="edit-attribute">
        <sqf:description>
          <sqf:title>Edit the @<name/> attribute value</sqf:title>
        </sqf:description>
        <sqf:user-entry name="value" default=".">
          <sqf:description>
            <sqf:title>Attribute value</sqf:title>
          </sqf:description>
        </sqf:user-entry>
        <sqf:replace node-type="attribute" target="ref" select="$value"/>
      </sqf:fix>
    </rule>
  </pattern>
</schema>
