<schema xmlns="http://purl.oclc.org/dsdl/schematron" queryBinding="xslt2"
        xmlns:sqf="http://www.schematron-quickfix.com/validator/process"
        xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
  <ns prefix="tei" uri="http://www.tei-c.org/ns/1.0"/>
  <ns prefix="xs" uri="http://www.w3.org/2001/XMLSchema"/>
  <ns prefix="fn" uri="tag:dmaus@dmaus.name,2019:Schematron:Workshop"/>
  <xsl:function name="fn:add-hashbang" as="xs:string*">
    <xsl:param name="tokens" as="xs:string*"/>
    <xsl:for-each select="$tokens">
      <xsl:value-of select="if (starts-with(., '#')) then . else concat('#', .)"/>
    </xsl:for-each>
  </xsl:function>
  <sqf:fixes>
    <sqf:fix id="unwrap">
      <sqf:description>
        <sqf:title>Unwrap the <name/> element</sqf:title>
      </sqf:description>
      <sqf:replace select="node()"/>
    </sqf:fix>
  </sqf:fixes>
  <pattern>
    <rule context="tei:foreign">
      <assert test="@xml:lang" sqf:fix="add-language">
        The foreign passage must have an indicator of the language in an @xml:lang attribute
      </assert>
      <sqf:fix id="add-language">
        <sqf:description>
          <sqf:title>Add the @xml:lang attribute</sqf:title>
        </sqf:description>
        <sqf:user-entry name="language">
          <sqf:description>
            <sqf:title>Language code</sqf:title>
          </sqf:description>
        </sqf:user-entry>
        <sqf:add node-type="attribute" target="xml:lang" select="$language"/>
      </sqf:fix>
    </rule>
    <rule context="tei:rs">
      <assert test="exists(@type)" sqf:fix="add-type">
        A referring string must have a @type attribute with a valid type.
      </assert>
      <assert test="empty(@type) or @type = ('place', 'person', 'org')" sqf:fix="fix-type unwrap">
        The value of a @type attribute of a referring string must either be 'place', 'person', or 'org'
      </assert>
      <sqf:fix id="add-type">
        <sqf:description>
          <sqf:title>Add the @type attribute</sqf:title>
        </sqf:description>
        <sqf:user-entry name="type">
          <sqf:description>
            <sqf:title>Type (either 'place', 'person', or 'org')</sqf:title>
          </sqf:description>
        </sqf:user-entry>
        <sqf:add node-type="attribute" target="type" select="$type"/>
      </sqf:fix>
      <sqf:fix id="fix-type">
        <sqf:description>
          <sqf:title>Change the @type attribute</sqf:title>
        </sqf:description>
        <sqf:user-entry name="type">
          <sqf:description>
            <sqf:title>Type (either 'place', 'person', or 'org')</sqf:title>
          </sqf:description>
        </sqf:user-entry>
        <sqf:add node-type="attribute" target="type" select="$type"/>
      </sqf:fix>
    </rule>
    <rule context="tei:rs/@ref">
      <let name="tokens" value="tokenize(., '\s+')"/>
      <assert test="exists($tokens)" sqf:fix="delete-attribute edit-attribute">
        The @ref attribute of a referring string must not be empty
      </assert>
      <assert test="every $token in $tokens satisfies starts-with($token, '#')" sqf:fix="remove-nohash add-hashbang edit-attribute">
        Every value in a @ref attribute of a referring string must start with a hashbang
      </assert>
      <sqf:fix id="remove-nohash">
        <sqf:description>
          <sqf:title>Remove values without a hashbang</sqf:title>
        </sqf:description>
        <sqf:replace node-type="attribute" target="ref" select="$tokens[starts-with(., '#')]"/>
      </sqf:fix>
      <sqf:fix id="add-hashbang">
        <sqf:description>
          <sqf:title>Add hashbang to values without one</sqf:title>
        </sqf:description>
        <sqf:replace node-type="attribute" target="ref" select="fn:add-hashbang($tokens)"/>
      </sqf:fix>
      <sqf:fix id="delete-attribute">
        <sqf:description>
          <sqf:title>Delete the @<name/> attribute</sqf:title>
        </sqf:description>
        <sqf:delete/>
      </sqf:fix>
      <sqf:fix id="edit-attribute">
        <sqf:description>
          <sqf:title>Edit the @<name/> attribute value</sqf:title>
        </sqf:description>
        <sqf:user-entry name="value" default=".">
          <sqf:description>
            <sqf:title>Attribute value</sqf:title>
          </sqf:description>
        </sqf:user-entry>
        <sqf:replace node-type="attribute" target="ref" select="$value"/>
      </sqf:fix>
    </rule>
  </pattern>
</schema>
